@@ -39,6 +39,7 @@ from solfege import gu
 from solfege import i18n
 from solfege import lessonfile
 from solfege import osutils
+from solfege import parsetree
 from solfege import reportlib
 from solfege import utils
 
@@ -345,7 +346,9 @@ class SolfegeApp(cfg.ConfigUtils):
         try:
             module = self._practise_lessonfile(filename)
         except (lessonfile.LessonfileParseException,
-                dataparser.DataparserException), e:
+                dataparser.DataparserException,
+                parsetree.ParseTreeException,
+                IOError), e:
             cleanup()
             gu.display_exception_message(e, lessonfile=filename)
             return
@@ -394,7 +397,7 @@ class SolfegeApp(cfg.ConfigUtils):
             # user tries to quit the program after parsing a bad lesson
             # file, we cannot call end_practise() without risking more
             # exceptions.
-            if g.m_t.m_P and getattr(g.m_t.m_P, 'header', None):
+            if g.m_t.m_P and hasattr(g.m_t.m_P, 'header'):
                 g.on_end_practise()
         try:
             cfg.sync()
