@@ -35,6 +35,8 @@ class Teacher(abstract.Teacher):
         abstract.Teacher.__init__(self, exname)
         self.lessonfileclass = lessonfile.ElembuilderLessonfile
         self.m_statistics = statistics.LessonStatistics(self)
+        for s in 'auto', 'show', 'play':
+            self.m_lessonfile_defs[s] = s
     def new_question(self):
         self.m_P.select_random_question()
         self.q_status = self.QSTATUS_NEW
@@ -78,7 +80,6 @@ class MultiButton(gtk.Button):
 class Gui(abstract.LessonbasedGui):
     def __init__(self, teacher):
         abstract.LessonbasedGui.__init__(self, teacher)
-        self.g_lesson_heading.hide()
         self.g_music_displayer = mpd.MusicDisplayer()
         self.practise_box.pack_start(self.g_music_displayer, False)
 
@@ -111,7 +112,8 @@ class Gui(abstract.LessonbasedGui):
         ##############
         # statistics #
         ##############
-        self.setup_statisticsviewer(statisticsviewer.PercentagesStatisticsViewer, "")
+        self.setup_statisticsviewer(statisticsviewer.PercentagesStatisticsViewer,
+                                   _("elembuilder"))
     def new_question(self, widget):
         self.g_answer.foreach(lambda w: w.destroy())
         self.m_users_answer = []
@@ -194,8 +196,8 @@ class Gui(abstract.LessonbasedGui):
             self.run_exception_handled(self.show_answer)
         self.g_backspace.set_sensitive(False)
     def on_start_practise(self):
-        super(Gui, self).on_start_practise()
         self.m_t.m_custom_mode = False # FIXME
+        super(Gui, self).on_start_practise()
         if self.m_t.m_P.header.elements == 'auto':
             v = []
             for question in self.m_t.m_P.m_questions:
@@ -204,8 +206,10 @@ class Gui(abstract.LessonbasedGui):
                         v.append(elem)
             def xcmp(a, b):
                 try:
-                    return cmp(self.m_t.m_P.blocklists['element'].index(a),
-                           self.m_t.m_P.blocklists['element'].index(b))
+                    try:
+                        return cmp(a['sort'], b['sort'])
+                    except KeyError:
+                        return cmp(a['name'], b['name'])
                 except ValueError:
                     return cmp(a, b)
             v.sort(xcmp)
