@@ -308,15 +308,16 @@ class FlashBar(gtk.Frame):
             if m:
                 varname = child[1:][:-1]
                 from solfege import lessonfilegui
-                w = lessonfilegui.new_labelobject(kwargs[varname])
-                w.show()
-                self.__content.pack_start(w, False)
-            else:
-                if child:
-                    label = gtk.Label(child)
-                    label.set_name("FlashBarLabel")
-                    self.__content.pack_start(label, False)
-                    label.show()
+                if isinstance(kwargs[varname], basestring):
+                    w = gtk.Label(kwargs[varname])
+                    w.set_name("FlashBarLabel")
+                else:
+                    w = lessonfilegui.new_labelobject(kwargs[varname])
+            elif child: # don't create label for empty string
+                w = gtk.Label(child)
+                w.set_name("FlashBarLabel")
+            self.__content.pack_start(w, False)
+            w.show()
         sx, sy = self.size_request()
         self.m_sx = max(sx, self.m_sx)
         self.m_sy = max(sy, self.m_sy)
@@ -449,18 +450,28 @@ class HarmonicProgressionLabel(gtk.HBox):
     def __init__(self, str):
         gtk.HBox.__init__(self)
         self.show()
-        self.m_xalign = 0.0
+        self.m_xalign = 0.5
         self.m_yalign = 0.5
         self.set_text(str)
     def set_alignment(self, xalign, yalign):
         self.m_xalign = xalign
         self.m_yalign = yalign
+        if xalign == 0.0:
+            self.g_prebox.hide()
+            self.g_postbox.show()
+        elif xalign == 0.5:
+            self.g_prebox.show()
+            self.g_postbox.show()
+        elif xalign == 1.0:
+            self.g_prebox.show()
+            self.g_postbox.hide()
     def set_text(self, str):
         for o in self.get_children():
             o.destroy()
         self.m_str = str
-        if self.m_xalign == 0.5:
-            self.pack_start(gtk.HBox())
+        self.g_prebox = gtk.HBox()
+        self.g_prebox.set_no_show_all(True)
+        self.pack_start(self.g_prebox)
         while self.m_str:
             T, A, B = self.get_next_token()
             if T == 'big' or T == 'err':
@@ -470,9 +481,11 @@ class HarmonicProgressionLabel(gtk.HBox):
             else:
                 assert T == 'one'
                 self.oneline(A)
-        if self.m_xalign == 0.5:
-            self.pack_start(gtk.HBox())
+        self.g_postbox = gtk.HBox()
+        self.g_postbox.set_no_show_all(True)
+        self.pack_start(self.g_postbox)
         self.show_all()
+        self.set_alignment(self.m_xalign, self.m_yalign)
     def get_next_token(self):
         m_re1 = re.compile("([^\(]+)", re.UNICODE)
         m_re2 = re.compile("\((\w*),\s*(\w*)\)", re.UNICODE)
@@ -490,7 +503,6 @@ class HarmonicProgressionLabel(gtk.HBox):
             self.m_str = self.m_str[len(m3.group()):]
             return "one", m3.groups()[0], None
         c = self.m_str[0]
-        self.m_str = self.m_str[1:]
         return "err", c, None
     def twoline(self, A, B):
         vbox = gtk.VBox()
@@ -766,7 +778,11 @@ class EditorDialogBase(object):
                 # to do handle string to int conversion in
                 # PractiseSheet.parse_file. Just let int() raise ValueException
                 # if the string in not an integer.
-                display_exception_message(e)
+                if 'msg1' in dir(e):
+                    solfege.win.display_error_message2(getattr(e, 'msg1', ''),
+                        getattr(e, 'msg2', ''))
+                else:
+                    display_exception_message(e)
         dialog.destroy()
     def get_save_as_dialog(self):
         dialog = gtk.FileChooserDialog(_("Save As..."), self,
