@@ -1,6 +1,6 @@
 # Solfege - free ear training software
 # vim: set fileencoding=utf-8 :
-# Copyright (C) 2007, 2008 Tom Cato Amundsen
+# Copyright (C) 2007, 2008, 2011 Tom Cato Amundsen
 # License is GPL, see file COPYING
 
 from __future__ import absolute_import
@@ -8,7 +8,7 @@ import gtk
 import unittest
 import time
 
-from solfege.testlib import I18nSetup
+from solfege.testlib import I18nSetup, TmpFileBase
 import solfege.lessonfile
 solfege.lessonfile._test_mode = True
 from solfege.lessonfile import *
@@ -16,42 +16,278 @@ from solfege import dataparser
 from solfege import mpd
 from solfege.mpd import mpdutils
 from solfege import cfg
+from solfege.parsetree import FunctionCall, LookupException
 
 
-class TestParser(unittest.TestCase):
+class MObjTest(TmpFileBase):
+    parserclass = QuestionsLessonfile
+    def _m(self, idx=0):
+        return self.p.m_questions[idx]['music']
+
+
+class TestParser(TmpFileBase):
+    parserclass = LessonfileCommon
     def test_bad_musictype(self):
-        s = 'question {\n' \
-            'music = music("c e g", "chor")\n' \
-            '}'
-        p = LessonfileCommon()
         try:
-            p.parse_string(s)
-        except dataparser.WrongArgumentCount, e:
+            self.do_file('question {\nmusic = music("c e g", "chor") }')
+        except FunctionCall.WrongArgumentCount, e:
             self.assertEquals('(line 1): question {\n'
-                              '(line 2): music = music("c e g", "chor")\n'
-                              '                  ^',
+                              '(line 2): music = music("c e g", "chor") }\n'
+                              '                        ^',
                               e.m_nonwrapped_text)
+        else:
+            self.fail("WrongArgumentCount not raised")
+    def test_bad_musictype_multiline(self):
+        try:
+            self.do_file('\n'.join([
+                'question {',
+                'music = music("c e g",',
+                '   "chor")',
+                '}']))
+        except FunctionCall.WrongArgumentCount, e:
+            self.assertEquals('(line 1): question {\n'
+                              '(line 2): music = music("c e g",\n'
+                              '                        ^',
+                              e.m_nonwrapped_text)
+        else:
+            self.fail("WrongArgumentCount not raised")
+    def test_exception_functioncall_no_args(self):
+        try:
+            self.do_file("a=1\n"
+                         "b = func() c=3")
+        except LookupException, e:
+            self.assertEquals("(line 1): a=1\n"
+                              "(line 2): b = func() c=3\n"
+                              "              ^",
+                              e.m_nonwrapped_text)
+        else:
+            self.fail("LookupException not raised")
+    def test_exception_functionsall_two_args(self):
+        try:
+            self.do_file("a=1\n"
+                         "b = func(3, 4) c=3")
+        except LookupException, e:
+            #self.assertEquals(e.m_token[TOKEN_STRING], 'func')
+            self.assertEquals("(line 1): a=1\n"
+                              "(line 2): b = func(3, 4) c=3\n"
+                              "              ^",
+                              e.m_nonwrapped_text)
+        else:
+            self.fail("LookupError not raised")
+    def test_exception_atom(self):
+        try:
+            self.do_file("a=b")
+        except LookupException, e:
+            #self.assertEquals(e.m_token[TOKEN_STRING], "b")
+            self.assertEquals("(line 1): a=b\n"
+                              "            ^",
+                              e.m_nonwrapped_text)
+        else:
+            self.fail("LookupError not raised")
+    def test_simplest(self):
+        p = self.do_file("a=yes")
+        self.assertEquals(p.m_globals['a'], True)
+    def test_tempo_tuple(self):
+        p = self.do_file("tempo = 240/4")
+        self.assertEquals(p.m_globals['tempo'], (240, 4))
     def test_gettext(self):
-        s = 'question {\n' \
+        p = self.do_file('question {\n' \
             '  name = _("chord|m7")\n' \
             '  iname = _i("chord|m7")\n' \
-            '}'
-        p = LessonfileCommon()
-        p.parse_string(s)
+            '  s = _("Major")\n' \
+            '}')
         self.assertEquals(p.m_questions[0].name, "chord|m7")
         self.assertEquals(p.m_questions[0].name.cval, "chord|m7")
         self.assertEquals(p.m_questions[0].iname, "m7")
         self.assertEquals(p.m_questions[0].iname.cval, "chord|m7")
+        self.assertEquals(p.m_questions[0].s, "Dur")
+        self.assertEquals(p.m_questions[0].s.cval, "Major")
+    def test_predef(self):
+        p = self.do_file("header { clef = treble }")
+        self.assert_(isinstance(p.header['clef'], unicode))
+        self.assertEquals(p.header['clef'], 'treble')
+    def test_module_names(self):
+        """
+        Test that all module names are allowed, even if they crash
+        with predefined functions or constants.
+        """
+        for modulename in ('rhythm', 'unknownmodule', 'yes'):
+            p = self.do_file("\n".join([
+                'header {',
+                '   module=%s',
+                '}',
+            ]) % modulename)
+            self.assert_(isinstance(p.header['module'], unicode))
+            self.assertEquals(p.header['module'], modulename)
+    def test_include(self):
+        self.add_file('element I { label = "I" }', 'progression_elements')
+        p = self.do_file('include("progression_elements")')
+        self.assert_('element' in p.blocklists)
+    def test_include_dont_overwrite_header_vars_but_adds_new(self):
+        self.add_file('header{ c = 2 d = 5 }', u'incfile')
+        p = self.do_file('\n'.join([
+            'header{ c = 4 }',
+            'include("incfile")',
+        ]))
+        self.assertEquals(p.header['c'], 4)
+        self.assertEquals(p.header['d'], 5)
+    def test_that_real_header_overwrites_previously_included(self):
+        self.add_file('header{ c = 2 }', 'incfile')
+        p = self.do_file('\n'.join([
+            'include("incfile")',
+            'header{ c = 4 }',
+            'include("incfile")',
+        ]))
+        self.assertEquals(p.header['c'], 4)
+    def test_load(self):
+        self.add_file('text', 'file-to-load')
+        p = self.do_file('s = load("file-to-load")')
+        self.assert_('s' in p.m_globals and p.m_globals['s'] == 'text')
+    def test_element_def(self):
+        p = self.do_file('element progI { label = progressionlabel("I") }')
+        self.assert_('label' in p.blocklists['element'][0])
+        self.assertEquals(p.blocklists['element'][0]['name'], 'progI')
+        self.assertEquals(p.m_globals['progI']['name'], 'progI')
+    def test_music_shortcut(self):
+        p = self.do_file('question { chord("c e g") }')
+        self.assert_('music' in p.m_questions[0])
+    def test_music_shortcut_string(self):
+        p = self.do_file('question { "c d e" }')
+        self.assert_('music' in p.m_questions[0])
+    def test_error_music_shortcut_string(self):
+        try:
+            self.do_file('question { "c d e"')
+        except dataparser.DataparserSyntaxError, e:
+            self.assertEquals(e.m_nonwrapped_text, "\n".join([
+                '(line 1): question { "c d e"',
+                '                            ^',
+            ]))
+        else:
+            self.fail("DataparserSyntaxError not raised")
+    def test_addition(self):
+        p = self.do_file("""
+              a = "en " + "to"
+              i = 3 + 5
+              tr1 = _("major")
+              tr2 = _("«%s» was not found")
+              tr3 = _("«%s» was not found") % "x"
+              tr4 = _("«%s» was not found") % 4
+              title = _("Is the interval flat, in tune or sharp? %s cent wrong") % 10
+              """)
+        self.assertEquals(p.m_globals['a'], "en to")
+        self.assertEquals(p.m_globals['i'], 8)
+        self.assertEquals(p.m_globals['tr1'], 'dur')
+        self.assertEquals(p.m_globals['tr2'], u'«%s» ble ikke funnet')
+        self.assertEquals(p.m_globals['tr3'], u'«x» ble ikke funnet')
+        self.assertEquals(p.m_globals['tr4'], u'«4» ble ikke funnet')
+    def test_absolute_import(self):
+        """
+        This test will import progression_elements and will show that
+        the program will import Solfege progression_elements, and not
+        the file in solfege/tests/lib/
+        """
+        p = LessonfileCommon()
+        p.parse_file(u"solfege/tests/lesson-files/test_absolute_import")
+        self.assertFalse('tests_progression_elements' in p.m_globals)
+        self.assertEquals(p.m_globals['progression_elements'].m_globals['I']['label'].m_labeltype, 'progressionlabel')
+        self.assertEquals(p.m_globals['progression_elements'].m_globals['I']['label'].m_labeldata, 'I')
+    def test_absolute_import_not_found_but_relative_exists(self):
+        """
+        absolute import will load the the file relative to the lesson
+        file if it does not exist in the standard exercises
+        """
+        p = LessonfileCommon()
+        self.assertFalse(os.path.exists("exercises/standard/lib/not_in_standard"))
+        p.parse_file(u"solfege/tests/lesson-files/test_absolute_import_not_found_load_relative")
+    def test_absolute_import_as(self):
+        """
+        This test will import progression_elements and will show that
+        the program will import Solfege progression_elements, and not
+        the file in solfege/tests/lib/
+        """
+        p = LessonfileCommon()
+        p.parse_file(u"solfege/tests/lesson-files/test_absolute_import_as")
+        self.assertFalse('tests_progression_elements' in p.m_globals)
+        self.assertEquals(p.m_globals['p'].m_globals['I']['label'].m_labeltype, 'progressionlabel')
+        self.assertEquals(p.m_globals['p'].m_globals['I']['label'].m_labeldata, 'I')
 
-class LessonfileTestCase(unittest.TestCase):
-    def _m(self, idx=0):
-        return self.p.m_questions[idx]['music']
+    def test_relative_import(self):
+        """
+        rimport will import the file in solfege/tests/lib, not the file
+        in the standard exercises
+        """
+        p = LessonfileCommon()
+        p.parse_file(u"solfege/tests/lesson-files/test_relative_import")
+        self.assertTrue('tests_progression_elements' in p.m_globals['progression_elements'].m_globals)
+    def test_relative_import_as(self):
+        """
+        rimport will import the file in solfege/tests/lib, not the file
+        in the standard exercises
+        """
+        p = LessonfileCommon()
+        p.parse_file(u"solfege/tests/lesson-files/test_relative_import_as")
+        self.assertTrue('tests_progression_elements' in p.m_globals['p'].m_globals)
+    def test_relative_import_not_found_load_standard(self):
+        """
+        rimport will load from the standard exercises if the file is not
+        found relative to the lesson file location.
+        """
+        p = LessonfileCommon()
+        p.parse_file(u"solfege/tests/lesson-files/test_relative_import_not_found_load_standard")
+        self.assertFalse(os.path.exists("solfege/tests/lib/jazz_progressions"))
 
-class TestMpdTransposable(unittest.TestCase):
+
+class TestParserTranslations(TmpFileBase, I18nSetup):
+    parserclass = LessonfileCommon
     def setUp(self):
+        TmpFileBase.setUp(self)
+        I18nSetup.setUp(self)
+    def tearDown(self):
+        TmpFileBase.tearDown(self)
+        I18nSetup.tearDown(self)
+    def test_istr_translations_in_file(self):
+        p = self.do_file("""
+           foo = "foo-C"
+           foo[nb] = "foo-nb"
+           # bar is not translated because the lang fr is not in the
+           # list of languages when the test suite is run
+           bar = "bar-C"
+           bar[fr] = "bar-fr"
+        """)
+        self.assertEquals(p.m_globals['foo'], u"foo-nb")
+        self.assertEquals(p.m_globals['foo'].cval, u"foo-C")
+        self.assertEquals(p.m_globals['bar'], u"bar-C")
+        self.assertEquals(p.m_globals['bar'].cval, u"bar-C")
+    def test_istr_translation_of_list(self):
+        try:
+            p = self.do_file('\n'.join(['foo = "foo-C", "fo2-C"',
+                                      'foo[nb] = "foo-nb", "fo2-nb"']))
+        except dataparser.CanOnlyTranslateStringsException, e:
+            self.assertEquals(e.m_nonwrapped_text, '\n'.join([
+                 '(line 1): foo = "foo-C", "fo2-C"',
+                 '(line 2): foo[nb] = "foo-nb", "fo2-nb"',
+                 '                    ^']))
+        else:
+            self.fail("CanOnlyTranslateStringsException not raised")
+    def test_istr_translation_of_int(self):
+        try:
+            p = self.do_file('\n'.join(['foo = 1',
+                                      'foo[nb] = 2']))
+        except dataparser.CanOnlyTranslateStringsException, e:
+            self.assertEquals(e.m_nonwrapped_text, '\n'.join([
+                 '(line 1): foo = 1',
+                 '(line 2): foo[nb] = 2',
+                 '                    ^']))
+        else:
+            self.fail("CanOnlyTranslateStringsException not raised")
+
+
+class TestMpdTransposable(TmpFileBase):
+    parserclass = LessonfileCommon
+    def setUp(self):
+        TmpFileBase.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header { random_transpose = "atonal", -5, 5 }
         question { music = chord("c e g") }
         """)
@@ -64,11 +300,11 @@ class TestMpdTransposable(unittest.TestCase):
         self.assertEquals("d fis a", self.p.m_questions[0]['music'].get_musicdata_transposed(self.p))
         self.assertEquals("c e g", self.p.m_questions[0]['music'].m_musicdata)
 
-class TestChord(unittest.TestCase):
+class TestChord(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header { random_transpose = no }
         question { music = chord("c e g") }
         """)
@@ -123,11 +359,11 @@ class TestChord(unittest.TestCase):
         self.p.m_transpose = mpd.MusicalPitch.new_from_notename("es'")
         self.assertEquals(self.p.m_questions[0]['music'].get_musicdata_transposed(self.p), "ees g bes")
 
-class TestMusic(unittest.TestCase):
+class TestMusic(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header { random_transpose = no }
         question { music = music("\staff{c''}\staff{e'}\staff{c}") }
         question { 
@@ -189,11 +425,11 @@ class TestMusic(unittest.TestCase):
         self.assertEquals(soundcard.synth.flush_testdata(1),
             "t30/4 p0:0 v0:100 n0:72 n0:64 n0:48 d1/4 o72 o64 o48")
 
-class TestMusic3(unittest.TestCase):
+class TestMusic3(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header { random_transpose = no }
         question { music = music3("\staff{c''}\staff{e'}\staff{c}") }
         question {
@@ -238,11 +474,12 @@ class TestVoiceCommon(unittest.TestCase):
         self.assertRaises(MusicObjectException, Rvoice("\\clef bass").get_first_pitch)
 
 
-class TestVoice(LessonfileTestCase):
+class TestVoice(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
         self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header { random_transpose = no }
         question { music = voice("c d e") }
         question { music = voice("< e") }
@@ -327,34 +564,15 @@ class TestVoice(LessonfileTestCase):
 
         else:
             self.fail("Test failed. No exception raised.")
-        return
-        try:
-            m = self._m(5).get_mpd_music_string(self.p)
-        except mpd.MpdException, e:
-            # We can not call complete_to_musicdata_coords becuase
-            # get_mpd_music_string promises to always raise exceptions
-            # relative to musicdata.
-            self.assertEquals(self._m(5).get_err_context(e, self.p),
-                "ERR f\n^^^")
-        else:
-            self.fail("Test failed. No exception raised.")
-        self.p.header.random_transpose = ('atonal', -4, 4)
-        try:
-            m = self._m(5).get_mpd_music_string(self.p)
-        except mpd.MpdException, e:
-            self.assertEquals(self._m(5).get_err_context(e, self.p),
-                "ERR f\n^^^")
-        else:
-            self.fail("Test failed. No exception raised.")
 
-class TestErrorHandling(unittest.TestCase):
+class TestErrorHandling(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
     def _dotest(self, musictypes, bad_code):
         for t in musictypes:
             for code, err_context in bad_code:
-                self.p = QuestionsLessonfile()
-                self.p.parse_string('''
+                self.do_file('''
                 header { random_transpose = no }
                 question { music = %s("%s") }
                 ''' % (t, code))
@@ -369,7 +587,7 @@ class TestErrorHandling(unittest.TestCase):
                     ec = question.music.get_err_context(e, self.p)
                     self.assertEquals(err_context, ec, "Music failed with musictype %s:\n%s !=\n%s" % (t, err_context, ec))
                 else:
-                    assert False, "No error found in code %s" % code
+                    self.fail("No exception raised in this code: %s" % code)
     def test_1(self):
         cfg.set_bool('config/override_default_instrument', False)
         self._test_1()
@@ -396,6 +614,35 @@ class TestErrorHandling(unittest.TestCase):
                      "ERR\n"
                      "^^^"),
                   ))
+    def test_barfull(self):
+        self._dotest(("music",),
+                   (("\\staff { c2. c }",
+                     "\\staff { c2. c }\n"
+                      "             ^"),
+                    ))
+        self._dotest(("voice",),
+                   (("c2. c",
+                     "c2. c\n"
+                     "    ^"),
+                    ))
+    def test_bad_clef(self):
+        self._dotest(("music",),
+                     (("\\staff { \\clef ERR",
+                      "\\staff { \\clef ERR\n"
+                        "         ^^^^^^^^^"),))
+        self._dotest(("voice", "rvoice"),
+                     (("\\clef ERR c d e",
+                       "\\clef ERR c d e\n"
+                        "^^^^^^^^^"),
+                      ("c \\clef ERR c d e",
+                       "c \\clef ERR c d e\n"
+                        "  ^^^^^^^^^"),
+                      ("\\time 4/4\n"
+                       "   c d \clef ERR3 e f\n"
+                       "  ",
+                       "   c d \clef ERR3 e f\n"
+                       "       ^^^^^^^^^^"),
+                        ))
     def test_satb(self):
         self._test_satb()
     def test_satb_3patches(self):
@@ -445,11 +692,11 @@ class TestErrorHandling(unittest.TestCase):
             ))
 
 
-class TestRvoice(LessonfileTestCase):
+class TestRvoice(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header { random_transpose = no }
         question { music = rvoice("c' d b c") }
         question { music = rvoice("[c'8 d] e16") }
@@ -466,7 +713,7 @@ class TestRvoice(LessonfileTestCase):
             "\\staff\\relative c'{\nc d b c\n}")
         self.p.header.random_transpose = (True,)
         self.assertEquals(self._m().get_mpd_music_string(self.p),
-            "\\staff\\transpose d'\\relative c'{ c d b c }")
+            "\\staff\\transpose d'\\relative c'{\nc d b c\n}")
         self.p.header.random_transpose = ['atonal', -5, 6]
         self.assertEquals(self._m().get_mpd_music_string(self.p),
             "\\staff\\relative d'{\nd e cis d\n}")
@@ -509,8 +756,10 @@ class TestRvoice(LessonfileTestCase):
                 "Bad input to the music object of type rvoice made\n"
                 "the parser fail to parse the following generated\n"
                 "music code:\n"
-                "\\staff\\transpose d'\\relative e{ < e }\n"
-                "                                    ^")
+                "\\staff\\transpose d'\\relative e{\n"
+                "< e\n"
+                "}\n"
+                "^")
         else:
             self.fail("Test failed. No exception raised.")
         m = self._m(5).get_mpd_music_string(self.p)
@@ -521,8 +770,10 @@ class TestRvoice(LessonfileTestCase):
                 "Bad input to the music object of type rvoice made\n"
                 "the parser fail to parse the following generated\n"
                 "music code:\n"
-                "\\staff\\transpose d'\\relative ERR{ ERR }\n"
-                "                   ^^^^^^^^^^^^^")
+                "\\staff\\transpose d'\\relative ERR{\n"
+                "                   ^^^^^^^^^^^^^\n"
+                "ERR\n"
+                "}")
         else:
             self.fail("Test failed. No exception raised.")
         m = self._m(6).get_mpd_music_string(self.p)
@@ -533,15 +784,19 @@ class TestRvoice(LessonfileTestCase):
                 "Bad input to the music object of type rvoice made\n"
                 "the parser fail to parse the following generated\n"
                 "music code:\n"
-                "\\staff\\transpose d'\\relative c{ c d e f\n"
-                "g ERR a }\n"
-                "  ^^^")
+                "\\staff\\transpose d'\\relative c{\n"
+                "c d e f\n"
+                "g ERR a\n"
+                "  ^^^\n"
+                "}")
+        else:
+            self.fail("Test failed. No exception raised.")
 
-class TestSatb(unittest.TestCase):
+class TestSatb(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header { random_transpose = no }
         question { music = satb("c''| e'| g | c") }
         question { key = "aes \\major" music = satb("c''|as'|es'|as") }
@@ -583,11 +838,11 @@ class TestSatb(unittest.TestCase):
         self.assertEquals(soundcard.synth.flush_testdata(),
             "t60/4 p0:0 v0:100 n62 d1/4 o62 n57 d1/4 o57 n54 d1/4 o54 n57 d1/4 o57 n50 d1/4 o50")
 
-class TestRhythm(unittest.TestCase):
+class TestRhythm(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
-        self.p = QuestionsLessonfile()
-        self.p.parse_string(r"""
+        self.do_file(r"""
         question { music = rhythm("d c8") }
         question { music = rhythm("c \time 3\4 c d") }
         """)
@@ -602,13 +857,12 @@ class TestRhythm(unittest.TestCase):
         self.assertRaises(mpd.InvalidNotenameException,
             question.music.get_mpd_music_string, self.p)
 
-class TestPercussion(unittest.TestCase):
+class TestPercussion(MObjTest):
     def setUp(self):
+        MObjTest.setUp(self)
         cfg.set_bool('config/override_default_instrument', False)
         self.p = QuestionsLessonfile()
-        self.p.parse_string("""
-        question { music = percussion("e16 f g4") }
-        """)
+        self.do_file('question { music = percussion("e16 f g4") }')
         self.p._idx = 0
     def test_play(self):
         question = self.p.m_questions[0]
@@ -617,12 +871,10 @@ class TestPercussion(unittest.TestCase):
             "t60/4 v9:100 P52 d1/16 o52 P53 d1/16 o53 P55 d1/4 o55")
 
 
-class TestWavfile(unittest.TestCase):
+class TestWavfile(MObjTest):
     def setUp(self):
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
-        question { music = wavfile("exercises/standard/lesson-files/share/fifth-small-293.33.wav") }
-        """)
+        MObjTest.setUp(self)
+        self.do_file('question { music = wavfile("exercises/standard/lesson-files/share/fifth-small-293.33.wav") }')
         self.p._idx = 0
     def test_play(self):
         question = self.p.m_questions[0]
@@ -637,12 +889,10 @@ class TestWavfile(unittest.TestCase):
                           "exercises/standard/lesson-files/share/fifth-small-293.33.wav")
         self.assertTrue(isinstance(question.music, Wavfile))
 
-class TestMidifile(unittest.TestCase):
+class TestMidifile(MObjTest):
     def setUp(self):
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
-        question { music = midifile("exercises/standard/lesson-files/share/fanfare.midi") }
-        """)
+        MObjTest.setUp(self)
+        self.do_file('question { music = midifile("exercises/standard/lesson-files/share/fanfare.midi") }')
         self.p._idx = 0
     def test_play(self):
         question = self.p.m_questions[0]
@@ -658,12 +908,10 @@ class TestMidifile(unittest.TestCase):
         self.assertTrue(isinstance(question.music, Midifile))
 
 
-class TestCmdline(unittest.TestCase):
+class TestCmdline(MObjTest):
     def setUp(self):
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
-        question { music = cmdline("/usr/bin/play exercises/standard/lesson-files/share/fifth-pure-220.00.wav") }
-        """)
+        MObjTest.setUp(self)
+        self.do_file('question { music = cmdline("/usr/bin/play exercises/standard/lesson-files/share/fifth-pure-220.00.wav") }')
         self.p._idx = 0
     def test_play(self):
         question = self.p.m_questions[0]
@@ -671,12 +919,10 @@ class TestCmdline(unittest.TestCase):
             question['music'].play(self.p, question)
             time.sleep(3)
 
-class TestCSound(unittest.TestCase):
+class TestCSound(MObjTest):
     def test_CSound(self):
-        self.p = QuestionsLessonfile()
-        self.p.parse_string('''
+        self.do_file('''
 header {
-    lesson_id = "csound-intonation-M3-5cent"
     module = idbyname
     help = "idbyname-intonation"
     title = "Test "
@@ -693,42 +939,40 @@ f1     0    4096 10   1  ; use GEN10 to compute a sine wave
   i1  +     1   10000  660   0.08    0.08
 e
  """
- music = csound(load("exercises/standard/lesson-files/share/sinus-ad.orc"), csound_sco)
+ music = csound(load("../../../exercises/standard/lesson-files/share/sinus-ad.orc"), csound_sco)
 }''')
         question = self.p.m_questions[0]
         if cfg.get_bool('testing/may_play_sound'):
             question['music'].play(self.p, question)
 
 
-class TestMma(unittest.TestCase):
+class TestMma(MObjTest):
     def test_1arg(self):
-        s = """
+        p = self.do_file("""
         question {
             music = mma("1 Cmaj7")
         }
-        """
-        p = QuestionsLessonfile()
-        p.parse_string(s)
+        """)
         q = p.m_questions[0]
         if cfg.get_bool('testing/may_play_sound'):
             q['music'].play(p, q)
     def test_2arg(self):
-        s = """
+        p = self.do_file("""
         question {
             music = mma("ballad", "1 Cmaj7")
         }
-        """
-        p = QuestionsLessonfile()
-        p.parse_string(s)
+        """)
         q = p.m_questions[0]
         if cfg.get_bool('testing/may_play_sound'):
             q['music'].play(p, q)
 
 
-class TestLessonfileHeader(I18nSetup):
+class TestLessonfileHeader(MObjTest, I18nSetup):
+    def setUp(self):
+        MObjTest.setUp(self)
+        I18nSetup.setUp(self)
     def test_untranslated_variable(self):
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header { untranslated = "jojo"
            translated = _("major")
         }
@@ -743,8 +987,7 @@ class TestLessonfileHeader(I18nSetup):
         self.assertEquals(self.p.header.translated, u"dur")
         self.assertEquals(self.p.header.translated.cval, u"major")
     def test_infile_translations_discard(self):
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         header {
            title = "C-locale header"
            title[et] = "et-locale header"
@@ -756,10 +999,12 @@ class TestLessonfileHeader(I18nSetup):
         self.assertEquals(self.p.header.title, "C-locale header")
         self.assertEquals(self.p.header.title.cval, "C-locale header")
 
-class TestLessonfileQuestion(I18nSetup):
+class TestLessonfileQuestion(MObjTest, I18nSetup):
+    def setUp(self):
+        MObjTest.setUp(self)
+        I18nSetup.setUp(self)
     def test_question_name(self):
-        self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.do_file("""
         question { name = "untranslated" }
         question { name = _("major") }
         question { # question has no name
@@ -776,7 +1021,8 @@ class TestLessonfileQuestion(I18nSetup):
         self.assertEquals(q.var, 1)
         self.assertEquals(q['var'], 1)
 
-class TestLessonfileMisc(unittest.TestCase):
+class TestLessonfileMisc(TmpFileBase):
+    parserclass = IdPropertyLessonfile
     def test_eq(self):
         c = Chord("a b c")
         d = Rvoice("a b c")
@@ -793,8 +1039,7 @@ class TestLessonfileMisc(unittest.TestCase):
             question { name = "one" }
             question { name = "two" inversion=2 }
             """
-        p = IdPropertyLessonfile()
-        p.parse_string(s)
+        p = self.do_file(s)
         # One question should be discarded because it does not have
         # the inversion variable
         self.assertEquals(len(p.m_discards), 1)
@@ -803,8 +1048,7 @@ class TestLessonfileMisc(unittest.TestCase):
         s = 'header { qprops = "name", "inversion" qprop_labels = "Name", "Inversion" }\n' \
             'question { name = "one" inversion=1} \n' \
             'question { name = "two" inversion=2 top=0 } '
-        p = IdPropertyLessonfile()
-        p.parse_string(s)
+        p = self.do_file(s)
         # No questions are discaded.
         # top is not a property since it is not added to qprops, so
         # thats why the first question is not discarded.
@@ -814,8 +1058,7 @@ class TestLessonfileMisc(unittest.TestCase):
         s = 'header { qprops = "name", "inversion", "toptone" qprop_labels = "Name", "Inversion", "Toptone" }\n' \
             'question { name = "one" inversion=1} \n' \
             'question { name = "two" inversion=2 top=0 } '
-        p = IdPropertyLessonfile()
-        p.parse_string(s)
+        p = self.do_file(s)
         # No questions are discarded, since they both contain the same
         # two properties, of the tree listed in qprops. The 'top' variable
         # in the second question does not affect what is or is not discarded
@@ -826,8 +1069,8 @@ class TestLessonfileMisc(unittest.TestCase):
             question { name = "one" }
             question { name = "two" inversion=2 }
             """
-        p = ChordLessonfile()
-        p.parse_string(s)
+        self.p = ChordLessonfile()
+        p = self.do_file(s)
         # One question should be discarded because it does not have
         # the inversion variable
         self.assertEquals(len(p.m_questions), 1)
@@ -835,8 +1078,8 @@ class TestLessonfileMisc(unittest.TestCase):
         s = 'header { qprops = "name", "inversion" qprop_labels = "Name", "Inversion" }\n' \
             'question { name = "one" inversion=1} \n' \
             'question { name = "two" inversion=2 top=0 } '
-        p = ChordLessonfile()
-        p.parse_string(s)
+        self.p = ChordLessonfile()
+        p = self.do_file(s)
         # No questions are discaded.
         # top is not a property since it is not added to qprops, so
         # thats why the first question is not discarded.
@@ -847,8 +1090,8 @@ class TestLessonfileMisc(unittest.TestCase):
         """
         s = 'header { qprops = "name" qprop_labels = "NAME" } ' \
             'question { name = "major" music = chord("c e g") } '
-        p = IdPropertyLessonfile()
-        p.parse_string(s)
+        self.p = IdPropertyLessonfile()
+        p = self.do_file(s)
         self.assertEquals(p.header.qprops, [u'name'])
     def test_nrandom(self):
         self._worker_Xrandom("nrandom")
@@ -865,8 +1108,8 @@ class TestLessonfileMisc(unittest.TestCase):
             style = %s(styles)
         }
         """ % (", ".join(['"%s"' % x for x in styles]), rfunc, rfunc)
-        p = QuestionsLessonfile()
-        p.parse_string(s)
+        self.p = QuestionsLessonfile()
+        p = self.do_file(s)
         for idx in range(2):
             st = p.m_questions[idx]['style']
             for x in range(20):
@@ -917,8 +1160,27 @@ class TestLabelObject(unittest.TestCase):
             ):
             self.assertEquals(chordname_markup_tokenizer(s), v)
 
+class TestOurLessonFiles(unittest.TestCase):
+    def test_test_requirement(self):
+        """
+        Check that all lesson files that has the header.test variable
+        also defines header.test_requirement
+        """
+        d = os.path.join("exercises", "standard", "lesson-files")
+        for fn in os.listdir(d):
+            if fn == 'Makefile':
+                continue
+            fullname = os.path.join(d, fn)
+            if os.path.isfile(fullname):
+                if fn.endswith("~") or fn.startswith("."):
+                    continue
+                f = parse_lesson_file_header(fullname)
+                self.failIf(f is None, "Parsing lesson header failed: %s" % fullname)
+                if 'test' in f.header:
+                    self.failIf('test_requirement' not in f.header, fullname)
 
 suite = unittest.makeSuite(TestParser)
+suite.addTest(unittest.makeSuite(TestParserTranslations))
 suite.addTest(unittest.makeSuite(TestMpdTransposable))
 suite.addTest(unittest.makeSuite(TestChord))
 suite.addTest(unittest.makeSuite(TestMusic))
@@ -939,3 +1201,4 @@ suite.addTest(unittest.makeSuite(TestLessonfileQuestion))
 suite.addTest(unittest.makeSuite(TestLessonfileMisc))
 suite.addTest(unittest.makeSuite(TestErrorHandling))
 suite.addTest(unittest.makeSuite(TestLabelObject))
+suite.addTest(unittest.makeSuite(TestOurLessonFiles))
